@@ -1,16 +1,9 @@
 #' Round, floor and ceiling for date-time objects
 #'
-#' @description \pkg{timechange} provides rounding to the nearest unit or
-#'   multiple of a unit. Units can be specified flexibly as strings; all common
-#'   abbreviations are supported - secs, min, mins, 2 minutes, 3 years, 2s, 1d
-#'   etc.
-#'
-#' @description `time_round()` rounds a date-time to the nearest value of the
-#'   specified time unit. For rounding date-times which is exactly halfway
-#'   between two consecutive units, the convention is to round up. Note that
-#'   this is in line with the behavior of R's [base::round.POSIXt()] function
-#'   but does not follow the convention of the base [base::round()] function
-#'   which "rounds to the even digit" per IEC 60559.
+#' @description \pkg{timechange} provides rounding to the nearest unit or multiple of a
+#'   unit with fractional support whenever makes sense. Units can be specified flexibly
+#'   as strings. All common abbreviations are supported - secs, min, mins, 2 minutes, 3
+#'   years, 2s, 1d etc.
 #'
 #' @section Civil Time vs Absolute Time rounding:
 #'
@@ -29,6 +22,14 @@
 #' to large precision errors due to the floating point representation of the
 #' POSIXct objects.
 #'
+#' @section Note on `time_round()`:
+#'
+#' For rounding date-times which is exactly halfway between two consecutive units,
+#' the convention is to round up. Note that this is in line with the behavior of R's
+#' [base::round.POSIXt()] function but does not follow the convention of the base
+#' [base::round()] function which "rounds to the even digit" per IEC 60559.
+#'
+#'
 #' @section Ceiling of `Date` objects:
 #'
 #' By default rounding up `Date` objects follows 3 steps:
@@ -53,28 +54,37 @@
 #'
 #' @name time_round
 #' @param time a date-time vector (`Date`, `POSIXct` or `POSIXlt`)
-#' @param unit a character string specifying a time unit or a multiple of a
-#'   unit. Valid base periods for civil time rounding are `second`, `minute`,
-#'   `hour`, `day`, `week`, `month`, `bimonth`, `quarter`, `season`, `halfyear`
-#'   and `year`. The only unit for absolute time rounding is `asecond`. Other
-#'   absolute units can be achieved with multiples of `asecond` ("60a", "3600a"
-#'   etc). See "Details" and examples. Arbitrary unique English abbreviations
-#'   are allowed. With one letter abbreviations are supported, including the
-#'   `strptime` formats "y", "m", "d", "M", "H", "S". Multi-unit rounding of
-#'   weeks is currently not supported.
-#' @param change_on_boundary If NULL (the default) don't change instants on the
-#'   boundary (`time_ceiling(ymd_hms('2000-01-01 00:00:00'))` is `2000-01-01
-#'   00:00:00`), but round up `Date` objects to the next boundary
-#'   (`time_ceiling(ymd("2000-01-01"), "month")` is `"2000-02-01"`). When
-#'   `TRUE`, instants on the boundary are rounded up to the next boundary. When
-#'   `FALSE`, date-time on the boundary are never rounded up (this was the
-#'   default for \pkg{lubridate} prior to `v1.6.0`. See section `Rounding Up
-#'   Date Objects` below for more details.
-#' @param week_start When unit is `weeks`, this is the first day of the
-#'   week. Defaults to 1 (Monday).
-#' @return An object of the same class as the input object. When input is a
-#'   `Date` object and unit is smaller than `day` a `POSIXct` object is
-#'   returned.
+#' @param unit a character string specifying a time unit or a multiple of a unit. Valid
+#'   base periods for civil time rounding are `second`, `minute`, `hour`, `day`, `week`,
+#'   `month`, `bimonth`, `quarter`, `season`, `halfyear` and `year`. The only unit for
+#'   absolute time rounding is `asecond`. Other absolute units can be achieved with
+#'   multiples of `asecond` ("60a", "3600a" etc). See "Details" and examples. Arbitrary
+#'   unique English abbreviations are allowed. One letter abbreviations follow
+#'   `strptime` formats "y", "m", "d", "M", "H", "S". Multi-unit rounding of weeks is
+#'   currently not supported.
+#'
+#'   Rounding for a unit is performed from the parent's unit origin. For example when
+#'   rounding to seconds origin is start of the minute. When rounding to days, origin is
+#'   first date of the month. See examples.
+#'
+#'   With fractional sub-unit (unit < 1) rounding with child unit is performed
+#'   instead. For example 0.5mins == 30secs, .2hours == 12min etc.
+#'
+#'   Please note that for fractions which don't match exactly to integer number of the
+#'   child units only the integer part is used for computation. For example .7days =
+#'   16.8hours will use 16 hours during the computation.
+#'
+#' @param change_on_boundary If NULL (the default) don't change instants on the boundary
+#'   (`time_ceiling(ymd_hms('2000-01-01 00:00:00'))` is `2000-01-01 00:00:00`), but
+#'   round up `Date` objects to the next boundary (`time_ceiling(ymd("2000-01-01"),
+#'   "month")` is `"2000-02-01"`). When `TRUE`, instants on the boundary are rounded up
+#'   to the next boundary. When `FALSE`, date-time on the boundary are never rounded up
+#'   (this was the default for \pkg{lubridate} prior to `v1.6.0`. See section `Rounding
+#'   Up Date Objects` below for more details.
+#' @param week_start When unit is `weeks`, this is the first day of the week. Defaults
+#'   to 1 (Monday).
+#' @return An object of the same class as the input object. When input is a `Date`
+#'   object and unit is smaller than `day` a `POSIXct` object is returned.
 #' @seealso [base::round()]
 #' @examples
 #'
@@ -159,6 +169,31 @@
 #' difftime(time_floor(x, "hour"), x)
 #' time_floor(x, "3600a") # "2014-11-02 01:00:00 EST"  - 25m
 #' difftime(time_floor(x, "a"), x)
+#'
+#' ## behavior on the boundary when rounding multi-units
+#'
+#' x <- as.POSIXct("2009-08-28 22:56:59.23", tz = "UTC")
+#' time_ceiling(x, "3.4 secs")  # "2009-08-28 22:57:03.4"
+#' time_ceiling(x, "50.5 secs")  # "2009-08-28 22:57:50.5"
+#' time_ceiling(x, "57 min")  # "2009-08-28 22:57:00"
+#' time_ceiling(x, "56 min")  # "2009-08-28 23:56:00"
+#' time_ceiling(x, "7h")  # "2009-08-29 07:00:00"
+#' time_ceiling(x, "7d")  # "2009-08-29 00:00:00"
+#' time_ceiling(x, "8d")  # "2009-09-09 00:00:00"
+#' time_ceiling(x, "8m")  # "2009-09-01 00:00:00"
+#' time_ceiling(x, "6m")  # "2010-01-01 00:00:00"
+#' time_ceiling(x, "7m")  # "2010-08-01 00:00:00"
+#'
+#' x <- as.POSIXct("2010-11-25 22:56:57", tz = "UTC")
+#' time_ceiling(x, "6sec") # "2010-11-25 22:57:00"
+#' time_ceiling(x, "60sec") # "2010-11-25 22:57:00"
+#' time_ceiling(x, "6min") # "2010-11-25 23:00:00"
+#' time_ceiling(x, "60min") # "2010-11-25 23:00:00"
+#' time_ceiling(x, "4h") # "2010-11-26 00:00:00"
+#' time_ceiling(x, "15d") # "2010-12-01 00:00:00"
+#' time_ceiling(x, "15d") # "2010-12-01 00:00:00"
+#' time_ceiling(x, "6m") # "2011-01-01 00:00:00"
+#'
 #' @export
 time_round <- function(time, unit = "second", week_start = getOption("timechange.week_start", 1)) {
   if (length(time) == 0L)
@@ -169,24 +204,28 @@ time_round <- function(time, unit = "second", week_start = getOption("timechange
   unit <- standardise_unit_name(parsed_unit$unit)
   validate_nunit(unit, n)
 
-  if (n == 1 && is.POSIXt(time) && unit == "second") {
-    ## special case for fast absolute time rounding
-    round.POSIXt(time, units = base_units[[unit]])
-  } else {
-    ct <- to_posixct(time)
-    above <- unclass(C_time_ceiling(ct, unit, n, week_start, TRUE))
-    mid <- unclass(ct)
-    below <- unclass(C_time_floor(ct, unit, n, week_start))
-    wabove <- (above - mid) <= (mid - below)
-    wabove <- !is.na(wabove) & wabove
-    below[wabove] <- above[wabove]
-    from_posixct(.POSIXct(below, tz = tz(time)), time,
-                 force_date = !unit %in% c("second", "minute", "hour"))
+  ct <- to_posixct(time)
+
+  ## special case for fast absolute time rounding
+  if (n == 1 && unit %in% c("day", "hour", "minute", "second", "asecond")) {
+    out <- round.POSIXt(ct, units = base_units[[unit]])
+    return(from_posixlt(out, time, force_date = unit != "hour"))
   }
+
+  ## FIXME: Behavior or this logic is likely slightly different from the above base
+  ## rounding around DST. It has to do with hard-coded post-pre values in ceiling and
+  ## floor.
+  above <- unclass(C_time_ceiling(ct, unit, n, week_start, TRUE))
+  mid <- unclass(ct)
+  below <- unclass(C_time_floor(ct, unit, n, week_start))
+  wabove <- (above - mid) <= (mid - below)
+  wabove <- !is.na(wabove) & wabove
+  below[wabove] <- above[wabove]
+
+  from_posixct(.POSIXct(below, tz = tz(time)), time,
+               force_date = !unit %in% c("hour", "minute", "second", "asecond"))
 }
 
-#' @description `time_floor` rounds down a date-time to the nearest lower
-#'   boundary of the specified time unit.
 #' @name time_round
 #' @export
 time_floor <- function(time, unit = "seconds", week_start = getOption("timechange.week_start", 1)) {
@@ -200,12 +239,10 @@ time_floor <- function(time, unit = "seconds", week_start = getOption("timechang
   validate_nunit(unit, n)
 
   from_posixct(C_time_floor(to_posixct(time), unit, n, week_start),
-               time, force_date = !unit %in% c("second", "minute", "hour"))
+               time, force_date = !unit %in% c("asecond", "second", "minute", "hour"))
 
 }
 
-#' @description`time_ceiling()` rounds up the date-time to the nearest boundary
-#'   of the specified time unit.
 #' @name time_round
 #' @export
 time_ceiling <- function(time, unit = "seconds",
