@@ -1,20 +1,28 @@
 context("Time Update")
 
-test_that("Non-finite date-times are handled correctly", {
-  expect_identical(unclass(time_update(.POSIXct(Inf), hour = 1)), Inf)
-  expect_identical(unclass(time_add(.POSIXct(-Inf), hour = 1)), -Inf)
-  expect_identical(unclass(time_add(.POSIXct(NA_real_), hour = 1)), NA_real_)
+test_that("Integer mode posixct input works", {
+  set.seed(100)
+  int <- seq(ymd("2020-08-13", tz = "UTC"), by = "hour", length.out = 50)
+  expect_true(storage.mode(int) == "integer")
+  dbl <- int
+  secs <- runif(length(int))
+  expect_equal(time_update(int, second = int), time_update(dbl, second = int))
+  expect_equal(time_update(int, second = as.integer(int)), time_update(dbl, second = as.integer(int)))
+})
 
-  expect_identical(unclass(time_add(.Date(Inf), day = 1)), Inf)
-  expect_identical(unclass(time_add(.Date(-Inf), day = 1)), -Inf)
-  expect_identical(unclass(time_add(.POSIXct(NA_real_), day = 1)), NA_real_)
+test_that("Non-finite date-times are handled correctly", {
+  expect_equal(time_update(.POSIXct(Inf), hour = 1), .POSIXct(Inf))
+  expect_equal(time_add(.POSIXct(-Inf), hour = 1), .POSIXct(-Inf))
+  expect_equal(time_add(.POSIXct(NA_real_), hour = 1), .POSIXct(NA_real_))
+  expect_equal(time_add(.POSIXct(NA_real_), day = 1), .POSIXct(NA_real_))
+  expect_equal(time_add(.Date(Inf), day = 1), .Date(Inf))
+  expect_equal(time_add(.Date(-Inf), day = 1), .Date(-Inf))
 })
 
 ## most of the tests are borrowed from lubridate
 
 test_that("update.Date returns a date object", {
   date <- as.Date("05/05/2010", "%m/%d/%Y")
-  expect_that(time_update(date, day = 1), is_a("Date"))
   expect_that(time_update(date, yday = 1), is_a("Date"))
   expect_that(time_update(date, mday = 1), is_a("Date"))
   expect_that(time_update(date, wday = 1), is_a("Date"))
@@ -56,7 +64,6 @@ test_that("update.POSIXlt returns a POSIXlt object", {
   expect_that(time_update(poslt, second = 1), is_a("POSIXlt"))
   expect_that(time_update(poslt, minute = 1), is_a("POSIXlt"))
   expect_that(time_update(poslt, hour = 1), is_a("POSIXlt"))
-  expect_that(time_update(poslt, day = 1), is_a("POSIXlt"))
   expect_that(time_update(poslt, yday = 1), is_a("POSIXlt"))
   expect_that(time_update(poslt, mday = 1), is_a("POSIXlt"))
   expect_that(time_update(poslt, wday = 1), is_a("POSIXlt"))
@@ -71,7 +78,6 @@ test_that("update.POSIXct returns a POSIXct object", {
   expect_that(time_update(posct, second = 1), is_a("POSIXct"))
   expect_that(time_update(posct, minute = 1), is_a("POSIXct"))
   expect_that(time_update(posct, hour = 1), is_a("POSIXct"))
-  expect_that(time_update(posct, day = 1), is_a("POSIXct"))
   expect_that(time_update(posct, yday = 1), is_a("POSIXct"))
   expect_that(time_update(posct, mday = 1), is_a("POSIXct"))
   expect_that(time_update(posct, wday = 1), is_a("POSIXct"))
@@ -119,9 +125,9 @@ test_that("update works with fractional second", {
   poslt <- ltutc("2010-02-03 13:45:59.234")
   posct <- ltutc("2010-02-03 13:45:59.323")
   expect_equal(time_update(poslt, year = 2002, minute = 3), ltutc("2002-02-03 13:03:59.234"))
-  expect_equal(time_update(poslt, day = 1, minute = 1, hour = 3), ltutc("2010-02-01 03:01:59.234"))
+  expect_equal(time_update(poslt, mday = 1, minute = 1, hour = 3), ltutc("2010-02-01 03:01:59.234"))
   expect_equal(time_update(poslt, year = 2002, minute = 3, second = 0.5), ltutc("2002-02-03 13:03:00.5"))
-  expect_equal(time_update(poslt, day = 1, minute = 1, hour = 3, second = 0.5), ltutc("2010-02-01 03:01:00.5"))
+  expect_equal(time_update(poslt, mday = 1, minute = 1, hour = 3, second = 0.5), ltutc("2010-02-01 03:01:00.5"))
 })
 
 ## test_that("update works with non-existent dates", {
@@ -200,16 +206,16 @@ test_that("Updates on ydays works correctly with leap year", {
 test_that("update performs roll overs correctly for Date objects", {
   date <- ymd("2010-05-05")
   expect_equal(time_update(date, second = 61), ymd_hms("2010-05-05 00:01:01"))
-  expect_equal(time_update(date, day = 29, second = 61), ymd_hms("2010-05-29 00:01:01"))
+  expect_equal(time_update(date, mday = 29, second = 61), ymd_hms("2010-05-29 00:01:01"))
 
   date <- ymd("2001-02-20")
-  expect_equal(time_update(date, day = 29, roll_month = "preday"), ymd("2001-02-28"))
-  expect_equal(time_update(date, day = 29, roll_month = "postday"), ymd("2001-03-01"))
-  expect_equal(time_update(date, day = 28, hour = 25, second = 2, roll_month = "postday"),
+  expect_equal(time_update(date, mday = 29, roll_month = "preday"), ymd("2001-02-28"))
+  expect_equal(time_update(date, mday = 29, roll_month = "postday"), ymd("2001-03-01"))
+  expect_equal(time_update(date, mday = 28, hour = 25, second = 2, roll_month = "postday"),
                ymd_hms("2001-03-01 01:00:02"))
-  expect_equal(time_update(date, day = 28, hour = 25, second = 2, roll_month = "preday"),
+  expect_equal(time_update(date, mday = 28, hour = 25, second = 2, roll_month = "preday"),
                ymd_hms("2001-02-28 01:00:02"))
-  expect_equal(time_update(date, day = 28, hour = 25, second = 2, roll_month = "boundary"),
+  expect_equal(time_update(date, mday = 28, hour = 25, second = 2, roll_month = "boundary"),
                ymd_hms("2001-03-01 00:00:00"))
 
 
@@ -282,16 +288,16 @@ test_that("update performs roll overs correctly for POSIXct objects", {
 test_that("update performs consecutive roll overs correctly for
   Date objects regardless of order", {
     expect_equal(time_update(ymd("2010-01-11"),
-                             month = 13, day = 32, hour = 25,
+                             month = 13, mday = 32, hour = 25,
                              minute = 61, second = 61),
                  ymd_hms("2011-01-31 02:02:01"))
     expect_equal(time_update(ymd("2010-01-11"),
-                             month = 13, day = 32, hour = 25,
+                             month = 13, mday = 32, hour = 25,
                              minute = 61, second = 61,
                              roll_month = "boundary"),
                  ymd_hms("2011-02-01 00:00:00"))
     expect_equal(time_update(ymd("2010-01-11"),
-                             month = 13, day = 32, hour = 25,
+                             month = 13, mday = 32, hour = 25,
                              minute = 61, second = 61,
                              roll_month = "NA"),
                  NA_POSIXct_)
@@ -300,7 +306,7 @@ test_that("update performs consecutive roll overs correctly for
 test_that("update performs consecutive roll overs correctly for POSIXlt objects", {
   posl <- as.POSIXlt("2010-11-01 00:00:00",
                      tz = "GMT", format = "%Y-%m-%d %H:%M:%S")
-  poslt <- time_update(posl, month = 13, day = 32, hour = 25,
+  poslt <- time_update(posl, month = 13, mday = 32, hour = 25,
                        minute = 61, second = 61,
                        roll_month = "skip")
   expect_equal(second(poslt), 1)
@@ -313,7 +319,7 @@ test_that("update performs consecutive roll overs correctly for POSIXlt objects"
   expect_equal(year(poslt), 2011)
   expect_equal(timechange:::tz(poslt), "GMT")
   poslt2 <- time_update(posl, second = 61, minute = 61, hour = 25,
-                        day = 32, month = 13, roll_month = "skip")
+                        mday = 32, month = 13, roll_month = "skip")
   expect_equal(second(poslt2), 1)
   expect_equal(minute(poslt2), 2)
   expect_equal(hour(poslt2), 2)
@@ -328,7 +334,7 @@ test_that("update performs consecutive roll overs correctly for POSIXlt objects"
 test_that("update performs consecutive roll overs correctly for POSIXct objects", {
   posc <- as.POSIXct("2010-11-01 00:00:00",
                      tz = "GMT", format = "%Y-%m-%d %H:%M:%S")
-  posct <- time_update(posc, month = 13, day = 32, hour = 25,
+  posct <- time_update(posc, month = 13, mday = 32, hour = 25,
                        minute = 61, second = 61, roll_month = "skip")
   expect_equal(second(posct), 1)
   expect_equal(minute(posct), 2)
@@ -340,7 +346,7 @@ test_that("update performs consecutive roll overs correctly for POSIXct objects"
   expect_equal(year(posct), 2011)
   expect_equal(timechange:::tz(posct), "GMT")
   posct2 <- time_update(posc, second = 61, minute = 61, hour = 25,
-                        day = 32, month = 13, roll_month = "skip")
+                        mday = 32, month = 13, roll_month = "skip")
   expect_equal(second(posct2), 1)
   expect_equal(minute(posct2), 2)
   expect_equal(hour(posct2), 2)
@@ -378,17 +384,17 @@ test_that("time_update roll_dst specs  work correctly", {
   repref <- ymd_hms("2022-10-30 01:00:00", tz = "Europe/Amsterdam")
   rep <- ymd_hms("2022-10-29 00:01:02", tz = "Europe/Amsterdam")
 
-  expect_equal(time_update(rep, day = 30, hour = 2, minute = 3, roll_dst = "pre"),
+  expect_equal(time_update(rep, mday = 30, hour = 2, minute = 3, roll_dst = "pre"),
                repref + 3600 + 3*60 + 2)
-  expect_equal(time_update(rep, day = 30, hour = 2, minute = 3, roll_dst = "boundary"),
+  expect_equal(time_update(rep, mday = 30, hour = 2, minute = 3, roll_dst = "boundary"),
                repref + 2*3600)
-  expect_equal(time_update(rep, day = 30, hour = 2, minute = 3, roll_dst = "post"),
+  expect_equal(time_update(rep, mday = 30, hour = 2, minute = 3, roll_dst = "post"),
                repref + 2*3600 + 3*60 + 2)
-  expect_equal(time_update(rep, day = 30, hour = 2, second = 3.35, roll_dst = "pre"),
+  expect_equal(time_update(rep, mday = 30, hour = 2, second = 3.35, roll_dst = "pre"),
                repref + 3600 + 60 + 3.35)
-  expect_equal(time_update(rep, day = 30, hour = 2, second = 3.35, roll_dst = "boundary"),
+  expect_equal(time_update(rep, mday = 30, hour = 2, second = 3.35, roll_dst = "boundary"),
                repref + 2*3600)
-  expect_equal(time_update(rep, day = 30, hour = 2, second = 3.35, roll_dst = "post"),
+  expect_equal(time_update(rep, mday = 30, hour = 2, second = 3.35, roll_dst = "post"),
                repref + 2*3600 + 60 + 3.35)
 
   ## DST gap    2022-03-27 01:00:00 CET  -- 2022-03-27 02:00:00 CEST
@@ -459,7 +465,7 @@ test_that("update handles vectors of dates", {
   date <- as.Date(poslt)
   expect_equal(second(time_update(poslt, second = 1)), c(1, 1, 1))
   expect_equal(second(time_update(posct, second = 1)), c(1, 1, 1))
-  expect_equal(day(time_update(date, day = 1)), c(1, 1, 1))
+  expect_equal(day(time_update(date, mday = 1)), c(1, 1, 1))
 })
 
 test_that("update handles vectors of dates and conformable vector of inputs", {
@@ -469,7 +475,7 @@ test_that("update handles vectors of dates and conformable vector of inputs", {
   date <- as.Date(poslt)
   expect_equal(second(time_update(poslt, second = c(1, 2, 3))), c(1, 2, 3))
   expect_equal(second(time_update(posct, second = c(1, 2, 3))), c(1, 2, 3))
-  expect_equal(day(time_update(date, day = c(1, 2, 3))), c(1, 2, 3))
+  expect_equal(day(time_update(date, mday = c(1, 2, 3))), c(1, 2, 3))
 })
 
 test_that("update handles single vector of inputs", {
@@ -478,26 +484,26 @@ test_that("update handles single vector of inputs", {
   date <- as.Date(poslt)
   expect_equal(second(time_update(poslt, second = c(1, 2, 3))), c(1, 2, 3))
   expect_equal(second(time_update(posct, second = c(1, 2, 3))), c(1, 2, 3))
-  expect_equal(day(time_update(date, day = c(1, 2, 3))), c(1, 2, 3))
+  expect_equal(day(time_update(date, mday = c(1, 2, 3))), c(1, 2, 3))
 })
 
-test_that("update handles conformable vectors of inputs", {
+test_that("update follows vctrs replication rules", {
   poslt <- as.POSIXlt("2010-03-10 01:59:59", tz = "UTC", format = "%Y-%m-%d %H:%M:%S")
   posct <- as.POSIXct(poslt)
   date <- as.Date(poslt)
-  expect_equal(second(time_update(poslt, second = c(1, 2), minute = c(1, 2, 3, 4))), c(1, 2, 1, 2))
-  expect_equal(second(time_update(posct, second = c(1, 2), minute = c(1, 2, 3, 4))), c(1, 2, 1, 2))
-  expect_equal(day(time_update(date, day = c(1, 2), month = c(1, 2, 3, 4))), c(1, 2, 1, 2))
+  expect_error(time_update(poslt, second = c(1, 2), minute = c(1, 2, 3, 4)), "Incompatible")
+  expect_error(time_update(posct, second = c(1, 2), minute = c(1, 2, 3, 4)), "Incompatible")
+  expect_error(time_update(date, mday = c(1, 2), month = c(1, 2, 3, 4)), "Incompatible")
 })
 
 test_that("update.POSIXct returns input of length zero when given input of length zero", {
   x <- structure(vector(mode = "numeric"), class = c("POSIXct", "POSIXt"))
-  expect_equal(time_update(x, day = 1), x)
+  expect_equal(time_update(x, mday = 1), x)
 })
 
 test_that("update.POSIXlt returns input of length zero when given input of length zero", {
   x <- as.POSIXlt(structure(vector(mode = "numeric"), class = c("POSIXct", "POSIXt")))
-  expect_equal(time_update(x, day = 1), x)
+  expect_equal(time_update(x, mday = 1), x)
 })
 
 test_that("update correctly works for yday", {
@@ -517,8 +523,8 @@ test_that("update correctly works for yday", {
 
 test_that("update errors on empty unit vectors", {
   y <- ymd_hms("2020-03-29 01:00:00", tz = "Asia/Beirut")
-  expect_error(time_update(y, minute = integer()), "Invalid size of 'minute' vector")
-  expect_error(time_update(y, hour = 1, minute = integer()), "Invalid size of 'minute' vector")
+  expect_error(time_update(y, minute = integer()), "Incompatible size of 'minute' vector")
+  expect_error(time_update(y, hour = 1, minute = integer()), "Incompatible size of 'minute' vector")
 })
 
 
@@ -526,3 +532,14 @@ test_that("updating Dates with 0 hms units produces POSIXct.", {
   expect_equal(time_update(ymd("2020-03-03"), hour = 0, second = 0), ymd("2020-03-03", tz = "UTC"))
   expect_equal(time_update(ymd("2020-03-03"), hour = NULL), ymd("2020-03-03"))
 })
+
+
+test_that("NAs propagate in update", {
+  expect_equal(time_update(ymd("2020-03-03"), year = NA), NA_Date_)
+  expect_equal(time_update(ymd("2020-03-03"), year = NA, second = 1), NA_POSIXct_)
+  expect_equal(time_update(ymd("2020-03-03"), second = NA), NA_POSIXct_)
+  expect_equal(time_update(ymd("2020-03-03"), year = c(2021, NA), second = 1),
+               c(ymd_hms("2021-03-03 00:00:01 UTC", NA)))
+  expect_equal(time_update(ymd("2020-03-03"), year = c(2021, NA), second = c(NA, 10)),
+               c(NA_POSIXct_, NA_POSIXct_))
+})
